package me.totalfreedom.discord;

import me.totalfreedom.discord.command.HelpCommand;
import me.totalfreedom.discord.command.ListCommand;
import me.totalfreedom.discord.command.TPSCommand;
import me.totalfreedom.discord.handling.CommandHandler;
import me.totalfreedom.discord.listener.AdminChatListener;
import me.totalfreedom.discord.listener.BukkitNative;
import me.totalfreedom.discord.listener.MinecraftListener;
import me.totalfreedom.totalfreedommod.api.Context;
import me.totalfreedom.totalfreedommod.api.TFD4JCommons;
import org.bukkit.plugin.java.JavaPlugin;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class TFD4J extends JavaPlugin
{
    private final Logger slf4j = LoggerFactory.getLogger("TFD4J");
    private Bot bot;
    private MinecraftListener mc;
    private AdminChatListener ac;
    private TFD4JCommons tfd4jcommons;
    private BukkitNative bn;
    private CommandHandler ch;

    @Override
    public void onEnable()
    {
        slf4j().info("Hello from TFD4J! Initializing our API implementation...");
        this.tfd4jcommons = new TFD4JCommonsImpl(this);

        slf4j().info("API successfully initialized! Initializing our bot...");
        this.bot = new Bot();

        slf4j().info("Bot successfully initialized! Registering the Bukkit Native listener...");
        this.bn = new BukkitNative(this);

        slf4j().info("Bukkit Native listener successfully registered! Registering the Discord4J Listeners...");
        this.mc = new MinecraftListener(this);
        this.ac = new AdminChatListener(this);
        mc.minecraftChatBound();
        ac.adminChatBound();

        slf4j().info("Discord4J listeners successfully registered! Registering the Command Handler...");
        this.ch = new CommandHandler(bot.getClient().getRestClient());

        slf4j().info("Command Handler successfully registered! Registering commands...");
        this.ch.registerCommand(new HelpCommand());
        this.ch.registerCommand(new ListCommand());
        this.ch.registerCommand(new TPSCommand());

        slf4j().info("Commands successfully registered! Providing context to TFM...");
        Context<TFD4JCommons> context = new Context<>(tfd4jcommons);
        bot.getTFM().getCommons().ag.setDiscordContext(context);
        bot.getTFM().getCommons().registerDiscord();

        slf4j().info("Context provided! TFD4J is now ready to go!");
    }

    @Override
    public void onDisable()
    {
        slf4j().info("Disconnecting the Discord bot...");
        bot.getClient()
                .onDisconnect()
                .doOnError(th -> slf4j().error("Error disconnecting the bot!", th))
                .doOnSuccess(v -> slf4j().info("Bot disconnected!"))
                .subscribe();
        slf4j().info("Goodbye from TFD4J!");
    }

    public Logger slf4j()
    {
        return slf4j;
    }

    public Bot getBot()
    {
        return bot;
    }

    public TFD4JCommons getImpl()
    {
        return tfd4jcommons;
    }

    public MinecraftListener getMinecraftListener()
    {
        return mc;
    }

    public AdminChatListener getAdminChatListener()
    {
        return ac;
    }
}
