package me.totalfreedom.discord.listener;

import discord4j.core.event.domain.message.MessageCreateEvent;
import discord4j.core.object.entity.Attachment;
import discord4j.core.object.entity.Guild;
import discord4j.core.object.entity.Member;
import discord4j.core.object.entity.Message;
import discord4j.core.object.entity.channel.TextChannel;
import me.totalfreedom.discord.Bot;
import me.totalfreedom.discord.TFD4J;
import me.totalfreedom.discord.util.SnowflakeEntry;
import me.totalfreedom.totalfreedommod.TotalFreedomMod;
import me.totalfreedom.totalfreedommod.config.ConfigEntry;
import me.totalfreedom.totalfreedommod.rank.Rank;
import me.totalfreedom.totalfreedommod.rank.Title;
import me.totalfreedom.totalfreedommod.util.FLog;
import me.totalfreedom.totalfreedommod.util.FUtil;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.TextComponent;
import net.kyori.adventure.text.event.ClickEvent;
import net.kyori.adventure.text.event.HoverEvent;
import net.kyori.adventure.text.format.NamedTextColor;
import org.bukkit.Bukkit;
import org.bukkit.entity.Player;

public class MinecraftListener
{
    private final TotalFreedomMod commons;
    private final Bot bot;
    private final TFD4J tfd4j;

    public MinecraftListener(TFD4J tfd4j)
    {
        this.tfd4j = tfd4j;
        this.bot = tfd4j.getBot();
        this.commons = bot.getTFM().getCommons();
    }

    public void minecraftChatBound()
    {
        tfd4j.getBot().getClient()
                .getEventDispatcher()
                .on(MessageCreateEvent.class)
                .filter(m -> m.getMember().orElse(null) != null)
                .filter(m -> !m.getMessage()
                        .getAuthor()
                        .orElseThrow(IllegalAccessError::new)
                        .getId()
                        .equals(tfd4j.getBot().getClient().getSelfId()))
                .filter(m -> m.getMessage()
                        .getChannel()
                        .blockOptional()
                        .orElseThrow(IllegalAccessError::new)
                        .getId()
                        .equals(SnowflakeEntry.chatChannelID))
                .filter(m ->
                {
                    Boolean b = m.getMessage()
                            .getChannel()
                            .map(c -> c instanceof TextChannel)
                            .block();
                    return b != null && b;
                }).subscribe(this::doMessageBodyDetails);
    }

    private void doMessageBodyDetails(MessageCreateEvent m)
    {
        Member member = m.getMember().orElseThrow();
        Message msg = m.getMessage();
        String tag = getDisplay(member);
        TextComponent[] emsg = {Component.empty()}; // We are using a single value arrays here to silence SonarLint's "return value never used" bullshit.

        emsg[0] = emsg[0].append(Component.text("[", NamedTextColor.DARK_GRAY));
        TextComponent[] inviteLink = {Component.text("Discord")};
        inviteLink[0] = inviteLink[0].color(NamedTextColor.DARK_AQUA);
        HoverEvent<Component> hoverEvent = HoverEvent.showText(Component.text("Click to join our Discord server!"));
        ClickEvent clickEvent = ClickEvent.openUrl(ConfigEntry.DISCORD_INVITE_LINK.getString());
        inviteLink[0] = inviteLink[0].hoverEvent(hoverEvent);
        inviteLink[0] = inviteLink[0].clickEvent(clickEvent);
        emsg[0] = emsg[0].append(inviteLink[0]);
        emsg[0] = emsg[0].append(Component.text("] ", NamedTextColor.DARK_GRAY));

        // Tag (if they have one)
        if (tag != null)
        {
            emsg[0] = emsg[0].append(Component.text(tag));
        }

        emsg[0] = emsg[0].append(Component.space());

        // User
        TextComponent[] user = {Component.text(FUtil.stripColors(member.getDisplayName()))};
        user[0] = user[0].color(NamedTextColor.RED);
        emsg[0] = emsg[0].append(user[0]);

        // Message
        emsg[0] = emsg[0].append(Component.text(": ", NamedTextColor.DARK_GRAY));
        emsg[0] = emsg[0].append(Component.text(FUtil.stripColors(msg.getContent()), NamedTextColor.WHITE));
        // Attachments
        if (!msg.getAttachments().isEmpty())
        {
            if (!msg.getContent().isEmpty())
                emsg[0] = emsg[0].append(Component.space());

            for (Attachment attachment : msg.getAttachments())
            {
                TextComponent[] media = {Component.text("[Media] ")};
                media[0] = media[0].color(NamedTextColor.YELLOW);
                HoverEvent<Component> hover = HoverEvent.showText(Component.text(attachment.getUrl()));
                ClickEvent click = ClickEvent.openUrl(attachment.getUrl());
                media[0] = media[0].clickEvent(click);
                media[0] = media[0].hoverEvent(hover);
                emsg[0] = emsg[0].append(media[0]);
            }
        }


        for (Player player : Bukkit.getOnlinePlayers())
        {
            if (TotalFreedomMod.getPlugin().pl.getData(player).doesDisplayDiscord())
            {
                player.sendMessage(emsg[0]);
            }
        }

        FLog.info(emsg[0].content(), true);
    }

    public String getDisplay(Member member)
    {
        Guild server = tfd4j.getBot().getGuildById().block();
        // Server Owner
        assert server != null;
        return member.getRoles().map(role ->
        {
            if (role.getId().equals(SnowflakeEntry.ownerRoleID))
            {
                return Title.OWNER.getColoredTag();
            } else if (role.getId().equals(SnowflakeEntry.developerRoleID))
            {
                return Title.DEVELOPER.getColoredTag();
            } else if (role.getId().equals(SnowflakeEntry.executiveRoleID))
            {
                return Title.EXECUTIVE.getColoredTag();
            } else if (role.getId().equals(SnowflakeEntry.assistantRoleID))
            {
                return Title.ASSTEXEC.getColoredTag();
            } else if (role.getId().equals(SnowflakeEntry.seniorRoleID))
            {
                return Rank.SENIOR_ADMIN.getColoredTag();
            } else if (role.getId().equals(SnowflakeEntry.adminRoleID))
            {
                return Rank.ADMIN.getColoredTag();
            } else if (role.getId().equals(SnowflakeEntry.builderRoleID))
            {
                return Title.MASTER_BUILDER.getColoredTag();
            } else
            {
                return null;
            }
        }).blockFirst();
    }
}

